/*
 * Copyright (c) 2008-2020, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hazelcast.internal.compatibility.cluster.impl;

import com.hazelcast.internal.cluster.impl.operations.AbstractJoinOperation;
import com.hazelcast.nio.ObjectDataInput;
import com.hazelcast.nio.ObjectDataOutput;

import java.io.IOException;

/**
 * Compatibility operation that is sent by a 3.x member to check if the
 * cluster name (group name) matches the WAN configuration.
 */
public class CompatibilityWanAuthorizationOp extends AbstractJoinOperation {
    private String groupName;
    private Boolean response = Boolean.TRUE;

    public CompatibilityWanAuthorizationOp() {
    }

    @Override
    public void run() {
        String clusterName = getNodeEngine().getConfig().getClusterName();
        response = groupName.equals(clusterName);
    }

    @Override
    public Object getResponse() {
        return response;
    }

    @Override
    protected void readInternal(ObjectDataInput in) throws IOException {
        groupName = in.readUTF();
        // group password is not checked
        in.readUTF();
    }

    @Override
    protected void writeInternal(ObjectDataOutput out) throws IOException {
        throw new UnsupportedOperationException(getClass().getName() + " should not be serialized!");
    }

    @Override
    public int getClassId() {
        return CompatibilityClusterDataSerializerHook.AUTHORIZATION;
    }
}
